/**********************************************************************************
* C Library for the display module families
* TFT2428T, TFT50D and TFT60D from XV-Display.de
*
* Section: Display_Interface_XMEGA.h
*
* Revision 1
*
* AVR and AVR Xmega microcontrollers,
* SoftwareSPI and HardwareSPI supported.
*
* Note: Some functions accesses the user flash memory
* and the micro sd card. First program flash.xvc into user
* flash memory and insert a micro sd card (FAT32) with example
* pictures. The package file flash.xvc (images and fonts)
* and example windows bitmap pictures are provided in this
* demo (see folder resources).
*
* Programming user flash memory:
* (1) insert micro sd card, formattet with FAT32 and flash.xvc
*     saved on it.
* (2) power on display module. programming process starts and
*     the status will be displayed.
* (3) if programming is completed, power off and remove micro
*     sd card.
*
* Recommended development environment:
*
* Atmel Studio 6
* ---> http://www.atmel.com/Microsite/atmel_studio6/default.aspx
* Developmentboard Nano from dev-tools.de.
* ---> http://www.dev-tools.de/index.php/development-boards/developmentboard-nano
*
* We invests time and resources providing this open source code,
* please support www.avr-praxis.de and open-source hardware
* by purchasing products from www.mikrocontroller-praxis.de!
*
* Written by Dirk Rodenhausen (dirk at avr-praxis.de)
*
* Our webpages:
* http://www.avr-praxis.de
* German discussion forum related to AVR microcontroller family.
* http://www.mikrocontroller-praxis.de
* Onlineshop where you can purchase the XV-Display display modules and
* the Dev-Tools Nano development board.
* http://www.dev-tools.de
* Further resources of products available from www.mikrocontroller-praxis.de.
*
*
* BSD license, all text above must be included in any redistribution.
* Please read the License.txt file.
* AVR is a registered trademark of Atmel corporation.
**********************************************************************************/

#ifndef __XVDISPLAY_XMEGA_H__
#define __XVDISPLAY_XMEGA_H__



static inline void _DisplayInit_Hardware(void)
{

  /***
  DISPLAY CONTROL SIGNALS
  ***/
  DISPLAY_CONTROL_PORT.OUTCLR = 1<<DISPLAY_SIGNAL_BUSY;          // BUSY: Input, pullup activated
  DISPLAY_CONTROL_PORT.DIRCLR = 1<<DISPLAY_SIGNAL_BUSY;
  DISPLAY_CONTROL_PORT.DISPLAY_PINCTRL_BUSY =  ( 0b011 << 3 );   // BUSY Pullup

  DISPLAY_CONTROL_PORT.OUTSET = 1<<DISPLAY_SIGNAL_CS;
  DISPLAY_CONTROL_PORT.DIRSET = 1<<DISPLAY_SIGNAL_CS;        // CS: Output, high

  #if (DISPLAY_USE_TOUCHPANEL == true)
      DISPLAY_CONTROL_PORT.OUTCLR = 1<<DISPLAY_SIGNAL_TINT;   // TINT: Input Pullup
      DISPLAY_CONTROL_PORT.DIRCLR = 1<<DISPLAY_SIGNAL_TINT;
      DISPLAY_CONTROL_PORT.DISPLAY_PINCTRL_TINT = ( 0b011 << 3 );  // TINT Pullup
  #endif

  /***
  SPI software interface
  ***/
  DISPLAY_SPI_PORT.OUTSET = (1<<DISPLAY_SPI_MOSI) | (1<<DISPLAY_SPI_SCK);  // MOSI, SCK: Output, high
  DISPLAY_SPI_PORT.DIRSET = (1<<DISPLAY_SPI_MOSI) | (1<<DISPLAY_SPI_SCK);
  
  DISPLAY_SPI_PORT.DIRCLR = 1<<DISPLAY_SPI_MISO;   // MISO: Input, high impedance state 
  DISPLAY_SPI_PORT.OUTCLR = 1<<DISPLAY_SPI_MISO;   

  #if (DISPLAY_SPI_MODE_HARDWARESPI == true)
    /***
    Hardware SPI
  
    Note: Maximum SPI frequency is 2MHz. Be sure to configure SPI module not at a higher
          frequency. 
    ***/

    #if ((DISPLAY_SPI_SPEED_FCPU_DIV != 2) && (DISPLAY_SPI_SPEED_FCPU_DIV != 4) && (DISPLAY_SPI_SPEED_FCPU_DIV != 8) \
	    && (DISPLAY_SPI_SPEED_FCPU_DIV != 16) && (DISPLAY_SPI_SPEED_FCPU_DIV != 32) && (DISPLAY_SPI_SPEED_FCPU_DIV != 64) \
	    && (DISPLAY_SPI_SPEED_FCPU_DIV != 128))

        #warning No valid definition of DISPLAY_SPI_SPEED_FCPU_DIV. Using F_CPU/128 for SPI clock! \
                 DISPLAY_SPI_SPEED_FCPU_DIV will be found in config.h
    #endif

    switch (DISPLAY_SPI_SPEED_FCPU_DIV)
	{
      case 2:    DISPLAY_SPI_MODULE.CTRL = SPI_CLK2X_bm;
	             break;
      case 4:    DISPLAY_SPI_MODULE.CTRL = 0;
	             break;
	  case 8:    DISPLAY_SPI_MODULE.CTRL = SPI_CLK2X_bm | (1 << SPI_PRESCALER_gp);
				 break;
      case 16:   DISPLAY_SPI_MODULE.CTRL = 1 << SPI_PRESCALER_gp;
                 break;
      case 32:   DISPLAY_SPI_MODULE.CTRL = SPI_CLK2X_bm | (2 << SPI_PRESCALER_gp);
				 break;
      case 64:   DISPLAY_SPI_MODULE.CTRL = 2 << SPI_PRESCALER_gp;
				 break;
      default:   DISPLAY_SPI_MODULE.CTRL = 3 << SPI_PRESCALER_gp;

    }
    
	DISPLAY_SPI_MODULE.CTRL |= SPI_ENABLE_bm | SPI_MODE_3_gc | SPI_MASTER_bm;        // SPI Enable, Slave, SPI Mode 3 
	
  #endif



}  




#if (DISPLAY_SPI_MODE_HARDWARESPI == true)


  /***
  Hardware SPI
  
  Note: Maximum SPI frequency is 2MHz. Be sure to configure SPI module not at a higher
        frequency. 
  ***/

  uint8_t _SPI_RW (uint8_t output)
  {

    DISPLAY_SPI_MODULE.DATA = output;

    while(!(DISPLAY_SPI_MODULE.STATUS & SPI_IF_bm)) { }

    return DISPLAY_SPI_MODULE.DATA;			
					
  }		

#else



  /***
  Software SPI
  
  Note: Maximum SPI frequency is 2MHz. Typical frequency achived by function below
        is 1,5MHz with a Xmega AVR at 32MHz systemclock. If using other systems, possibly
		you have to insert delays, to slow down SPI frequency.
  ***/

  uint8_t _SPI_RW (uint8_t output)
  {
    uint8_t input=0;
    uint8_t i;
	
    for (i=0; i<8; i++)
    {
      DISPLAY_SPI_PORT.OUTCLR = 1<<DISPLAY_SPI_SCK;    // SCK low

      if (output & 0x80)
        DISPLAY_SPI_PORT.OUTSET = 1<<DISPLAY_SPI_MOSI; // MOSI high
      else
        DISPLAY_SPI_PORT.OUTCLR = 1<<DISPLAY_SPI_MOSI; // MOSI low

      DISPLAY_SPI_PORT.OUTSET = 1<<DISPLAY_SPI_SCK;    // SCK high

      input <<= 1;          // Input MISO

      if (DISPLAY_SPI_PORT.IN & (1<<DISPLAY_SPI_MISO))
  	    input |= 1;
      else
	    input &= ~1;

      output <<= 1;

    }
	
    return input;							//return the byte clocked in from SPI slave
  }		



#endif

#endif

